# ollama.py
import os, sys
from dotenv import load_dotenv
from langchain_community.chat_models import ChatOllama
from langchain_community.llms import Ollama
from langchain_core.messages import HumanMessage, SystemMessage
from langchain_core.prompts import ChatPromptTemplate, MessagesPlaceholder
from langchain_core.output_parsers import StrOutputParser
from ..llms import LLMs

class OllamaLLM(LLMs):
    name = "ollama"  # Class variable
    model = ""  # Class variable
    temperature = 0.2  # Class variable
    max_tokens = None  # Class variable

    def __init__(self, model, chat=True, model_auth=None, **model_kwargs):
        super().__init__()

        self.model = model
        self.model_kwargs = model_kwargs  # Store model kwargs

        # AUTHENTICATION
        try:
            if(model_auth):
                self.auth(model)
            else:
                self.auth()
        except Exception as e:
            print(f"Error during authentication: {e}")

        # Initialize the output parser here if it's going to be static
        self.output_parser = StrOutputParser()

        # If the prompt template is static, initialize it here
        try:
            self.prompt_template = ChatPromptTemplate.from_messages([
                SystemMessage(content="You are a helpful assistant. Answer all questions to the best of your ability."),
                MessagesPlaceholder(variable_name="messages"),
            ])
        except Exception as e:
            print(f"Error initializing prompt template: {e}")
            sys.exit(1)

        try:
            self.updateLLM(chat)
        except Exception as e:
            print(f"Error updating LLM: {e}")
            sys.exit(1)

    def updateLLM(self, chat=True):
        try:
            base_url = os.environ.get("OLLAMA_BASE_URL")
            self.llm = (ChatOllama if chat else Ollama)(
                model=self.model,
                base_url=base_url,
                temperature=self.temperature,
                num_predict=self.max_tokens,
                **self.model_kwargs
            )

            self.chain = self.prompt_template | self.llm | self.output_parser
        except Exception as e:
            print(f"Error initializing LLM or chain: {e}")
            sys.exit(1)

    @classmethod
    def auth(cls, model=None):
        # Ollama does not have auth - however it needs a "base_url" via OLLAMA_BASE_URL
        # By default it will assume "localhost", but we are going to use this as a requirement
        try:
            if(model):
                auth_status = load_dotenv(f".llms/ollama-{model}", override=True)
            else:
                auth_status = load_dotenv(".llms/ollama")
            if not auth_status:
                if not os.environ.get("OLLAMA_BASE_URL"):
                    raise EnvironmentError("`OLLAMA_BASE_URL` env variable not defined, and `.llms/ollama` not available")

        except Exception as e:
            print(f"Error loading authentication for Ollama: {e}")
            sys.exit(1)

    def run(self, prompt):
        try:
            response = self.chain.invoke({
                "messages": [
                    HumanMessage(content=prompt),
                ],
            })
            return response
        except Exception as e:
            print(f"Error invoking the chain: {e}")
            return None



# aws.py
import os, sys, boto3
from dotenv import load_dotenv
from langchain_aws import ChatBedrock
from langchain_aws import BedrockLLM
from langchain_core.messages import HumanMessage, SystemMessage
from langchain_core.prompts import ChatPromptTemplate, MessagesPlaceholder
from langchain_core.output_parsers import StrOutputParser
from ..llms import LLMs

class AWS(LLMs):
    name = "aws" # Class variable
    model = "" # Class variable
    temperature = 0.2  # Class variable

    def __init__(self, **model_kwargs):
        super().__init__()

        self.model_kwargs = model_kwargs  # Store model kwargs

        # AUTHENTICATION
        try:
            self.auth()
        except Exception as e:
            print(f"Error during authentication: {e}")

        # Initialize the output parser here if it's going to be static
        self.output_parser = StrOutputParser()

        # If the prompt template is static, initialize it here
        try:
            self.prompt_template = ChatPromptTemplate.from_messages([
                SystemMessage(content="You are a helpful assistant. Answer all questions to the best of your ability."),
                MessagesPlaceholder(variable_name="messages"),
            ])
        except Exception as e:
            print(f"Error initializing prompt template: {e}")
            sys.exit(1)

        try:
            self.updateLLM()
        except Exception as e:
            print(f"Error updating LLM: {e}")
            sys.exit(1)

    def updateLLM(self, chat=True):
        try:
            ACCESS_KEY = os.environ.get("AWS_ACCESS_KEY_ID")
            SECRET_KEY = os.environ.get("AWS_SECRET_ACCESS_KEY")
            REGION = os.environ.get("AWS_REGION")
            session = boto3.Session(aws_access_key_id=ACCESS_KEY, aws_secret_access_key=SECRET_KEY)
            client = session.client("bedrock-runtime", region_name=REGION)

            self.llm = (ChatBedrock if chat else BedrockLLM)(
                client=client,
                region_name=REGION,
                model_id=self.model,
                model_kwargs=self.model_kwargs
            )

            self.chain = self.prompt_template | self.llm | self.output_parser
        except Exception as e:
            print(f"Error initializing LLM or chain: {e}")
            sys.exit(1)

    @classmethod
    def auth(cls, model=None):
        # AWS Bedrock auth happens here
        try:
            if(model):
                auth_status = load_dotenv(f".llms/aws-{model}", override=True)
            else:
                auth_status = load_dotenv(".llms/aws")
            if not auth_status:
                if not (
                    os.environ.get("AWS_ACCESS_KEY_ID") and
                    os.environ.get("AWS_SECRET_ACCESS_KEY") and
                    os.environ.get("AWS_REGION")
                ):
                    raise EnvironmentError("`AWS_ACCESS_KEY_ID` or `AWS_SECRET_ACCESS_KEY` or `AWS_REGION` env variables not defined, and `.llms/aws` not available")

        except Exception as e:
            print(f"Error loading authentication for AWS: {e}")
            sys.exit(1)

    def run(self, prompt):
        try:
            response = self.chain.invoke({
                "messages": [
                    HumanMessage(content=prompt),
                ],
            })
            return response
        except Exception as e:
            print(f"Error invoking the chain: {e}")
            return None



# azure/__init__.py
from .gpt35turbo import AzureGPT35Turbo as azure_gpt_35_turbo
from .gpt35turbo16k import AzureGPT35Turbo16k as azure_gpt_35_turbo_16k
from .gpt4turbo import AzureGPT4Turbo as azure_gpt_4_turbo
from .gpt4o import AzureGPT4o as azure_gpt_4o
from .gpt4 import AzureGPT4 as azure_gpt_4
from .gpt432k import AzureGPT432k as azure_gpt_4_32k
from .azure import AzureOpenAI

__all__ = ['azure_gpt_35_turbo', 'azure_gpt_35_turbo_16k', 'azure_gpt_4_turbo', 'azure_gpt_4o', 'azure_gpt_4', 'azure_gpt_4_32k']



# tests/test_sample.py
import unittest
from llms import some_function  # Import the function you want to test from your module

class TestSample(unittest.TestCase):

    def test_some_function(self):
        result = some_function()
        self.assertEqual(result, expected_value)

if __name__ == '__main__':
    unittest.main()



# aws/__init__.py
from .claude3haiku import Claude3Haiku as claude_3_haiku
from .claude3sonnet import Claude3Sonnet as claude_3_sonnet
from .claude3opus import Claude3Opus as claude_3_opus
from .claude35sonnet import Claude35Sonnet as claude_35_sonnet
from .claude1instant import Claude1Instant as claude_1_instant
from .claude2 import Claude2 as claude_2
from .llama270b import Llama270b as llama2_70b
from .llama38binstruct import Llama38bInstruct as llama3_8b_instruct
from .llama370binstruct import Llama370bInstruct as llama3_70b_instruct
from .mistral7binstruct import Mistral7bInstruct as mistral_7b_instruct
from .mistrallarge import MistralLarge as mistral_large
from .mistralsmall import MistralSmall as mistral_small
from .mixtral8x7binstruct import Mixtral8x7bInstruct as mixtral_8x7b_instruct
from .coherecommand14 import CohereCommand14 as cohere_command_14
from .coherecommandlight14 import CohereCommandLight14 as cohere_command_light_14
#from .coherecommandr import CohereCommandR as coherecommand_r
#from .coherecommandrplus import CohereCommandRPlus as coherecommand_r_plus
from .j2mid import J2Mid as j2_mid
from .j2ultra import J2Ultra as j2_ultra
#from .j2jambainstruct import J2JambaInstruct as j2_jamba_instruct
from .titanlitev1 import TitanLiteV1 as titan_lite_v1
from .titanexpressv1 import TitanExpressV1 as titan_express_v1
from .titanpremierv1 import TitanPremierV1 as titan_premier_v1
from .aws import AWS

__all__ = ['claude_3_haiku', 'claude_3_sonnet', 'claude_3_opus', 'claude_35_sonnet', 'claude_1_instant', 'claude_2', 'llama2_70b', 'llama3_8b_instruct', 'llama3_70b_instruct', 'mistral_7b_instruct', 'mistral_large', 'mistral_small', 'mixtral_8x7b_instruct', 'cohere_command_14', 'cohere_command_light_14', 'j2_mid', 'j2_ultra', 'titan_lite_v1', 'titan_express_v1', 'titan_premier_v1']



# llms/__init__.py
from .llms import LLMs as llms
from .openai import OpenAI as openai
from .openai.gpt35turbo import GPT35Turbo as gpt_35_turbo
from .openai.gpt35turbo16k import GPT35Turbo16k as gpt_35_turbo_16k
from .openai.gpt4turbo import GPT4Turbo as gpt_4_turbo
from .openai.gpt4o import GPT4o as gpt_4o
from .openai.gpt4omini import GPT4oMini as gpt_4o_mini
from .openai.gpt4 import GPT4 as gpt_4
from .openai.gpt432k import GPT432k as gpt_4_32k
from .azure import AzureOpenAI as azure
from .azure.gpt35turbo import AzureGPT35Turbo as azure_gpt_35_turbo
from .azure.gpt35turbo16k import AzureGPT35Turbo16k as azure_gpt_35_turbo_16k
from .azure.gpt4turbo import AzureGPT4Turbo as azure_gpt_4_turbo
from .azure.gpt4o import AzureGPT4o as azure_gpt_4o
from .azure.gpt4 import AzureGPT4 as azure_gpt_4
from .azure.gpt432k import AzureGPT432k as azure_gpt_4_32k
from .aws import AWS as aws
from .aws.claude3haiku import Claude3Haiku as claude_3_haiku
from .aws.claude3sonnet import Claude3Sonnet as claude_3_sonnet
from .aws.claude3opus import Claude3Opus as claude_3_opus
from .aws.claude35sonnet import Claude35Sonnet as claude_35_sonnet
from .aws.claude1instant import Claude1Instant as claude_1_instant
from .aws.claude2 import Claude2 as claude_2
from .aws.llama270b import Llama270b as llama2_70b
from .aws.llama38binstruct import Llama38bInstruct as llama3_8b_instruct
from .aws.llama370binstruct import Llama370bInstruct as llama3_70b_instruct
from .aws.mistral7binstruct import Mistral7bInstruct as mistral_7b_instruct
from .aws.mistrallarge import MistralLarge as mistral_large
from .aws.mistralsmall import MistralSmall as mistral_small
from .aws.mixtral8x7binstruct import Mixtral8x7bInstruct as mixtral_8x7b_instruct
from .aws.coherecommand14 import CohereCommand14 as cohere_command_14
from .aws.coherecommandlight14 import CohereCommandLight14 as cohere_command_light_14
#from .aws.coherecommandr import CohereCommandR as coherecommand_r
#from .aws.coherecommandrplus import CohereCommandRPlus as coherecommand_r_plus
from .aws.j2mid import J2Mid as j2_mid
from .aws.j2ultra import J2Ultra as j2_ultra
#from .aws.j2jambainstruct import J2JambaInstruct as j2_jamba_instruct
from .aws.titanlitev1 import TitanLiteV1 as titan_lite_v1
from .aws.titanexpressv1 import TitanExpressV1 as titan_express_v1
from .aws.titanpremierv1 import TitanPremierV1 as titan_premier_v1
from .google import Google as google
from .google.geminipro1 import GeminiPro1 as gemini_pro_1
from .google.geminipro15 import GeminiPro15 as gemini_pro_1_5
from .google.geminiflash15 import GeminiFlash15 as gemini_flash_1_5
from .google.bison import Bison as bison
from .google.bison32k import Bison32k as bison_32k
from .ollama import OllamaLLM as ollama

_all__ = ['llms', 'openai', 'gpt_35_turbo', 'gpt_35_turbo_16k', 'gpt_4_turbo', 'gpt_4o', 'gpt_4o_mini', 'gpt_4', 'gpt_4_32k', 'azure', 'azure_gpt_35_turbo', 'azure_gpt_35_turbo_16k', 'azure_gpt_4_turbo', 'azure_gpt_4o', 'azure_gpt_4', 'azure_gpt_4_32k', 'aws', 'claude_3_haiku', 'claude_3_sonnet', 'claude_3_opus', 'claude_35_sonnet', 'claude_1_instant', 'claude_2', 'llama2_70b', 'llama3_8b_instruct', 'llama3_70b_instruct', 'mistral_7b_instruct', 'mistral_large', 'mistral_small', 'mixtral_8x7b_instruct', 'cohere_command_14', 'cohere_command_light_14', 'j2_mid', 'j2_ultra', 'google', 'gemini_pro_1', 'gemini_pro_1_5', 'gemini_flash_1_5', 'bison', 'bison_32k', 'ollama']



# azure.py
import os, sys
from dotenv import load_dotenv
from langchain_openai import AzureChatOpenAI
from langchain_core.messages import HumanMessage, SystemMessage
from langchain_core.prompts import ChatPromptTemplate, MessagesPlaceholder
from langchain_core.output_parsers import StrOutputParser
from ..llms import LLMs

class AzureOpenAI(LLMs):
    name = "azure"
    model = ""  # Class variable
    temperature = 0.2  # Class variable
    max_tokens = None  # Class variable

    def __init__(self, **model_kwargs):
        super().__init__()

        self.model_kwargs = model_kwargs  # Store model kwargs

        # AUTHENTICATION
        try:
            self.auth()
        except Exception as e:
            print(f"Error during authentication: {e}")

        # Initialize the output parser here if it's going to be static
        self.output_parser = StrOutputParser()

        # If the prompt template is static, initialize it here
        try:
            self.prompt_template = ChatPromptTemplate.from_messages([
                SystemMessage(content="You are a helpful assistant. Answer all questions to the best of your ability."),
                MessagesPlaceholder(variable_name="messages"),
            ])
        except Exception as e:
            print(f"Error initializing prompt template: {e}")
            sys.exit(1)

        try:
            self.updateLLM()
        except Exception as e:
            print(f"Error updating LLM: {e}")
            sys.exit(1)

    def updateLLM(self):
        try:
            AZURE_OPENAI_API_VERSION = os.environ.get("AZURE_OPENAI_API_VERSION")
            AZURE_OPENAI_ENDPOINT = os.environ.get("AZURE_OPENAI_ENDPOINT")
            AZURE_OPENAI_API_KEY = os.environ.get("AZURE_OPENAI_API_KEY")
            self.llm = AzureChatOpenAI(
                api_key=AZURE_OPENAI_API_KEY,
                api_version=AZURE_OPENAI_API_VERSION,
                azure_endpoint=AZURE_OPENAI_ENDPOINT,
                azure_deployment=self.model,
                temperature=self.temperature,
                max_tokens=self.max_tokens,
                model_kwargs=self.model_kwargs
            )

            self.chain = self.prompt_template | self.llm | self.output_parser
        except Exception as e:
            print(f"Error initializing LLM or chain: {e}")
            sys.exit(1)

    @classmethod
    def auth(cls, model=None):
        # Azure OpenAI auth happens here
        try:
            if(model):
                auth_status = load_dotenv(f".llms/azure-{model}", override=True)
            else:
                auth_status = load_dotenv(".llms/azure")
            if not auth_status:
                if not (
                    os.environ.get("AZURE_OPENAI_ENDPOINT") and
                    os.environ.get("AZURE_OPENAI_API_KEY") and
                    os.environ.get("AZURE_OPENAI_API_VERSION")
                ):
                    raise EnvironmentError("`AZURE_OPENAI_ENDPOINT` or `AZURE_OPENAI_API_KEY` or `AZURE_OPENAI_API_VERSION` env variables not defined, and `.llms/azure` not available")

        except Exception as e:
            print(f"Error loading authentication for Azure OpenAI: {e}")
            sys.exit(1)

    def run(self, prompt):
        try:
            response = self.chain.invoke({
                "messages": [
                    HumanMessage(content=prompt),
                ],
            })
            return response
        except Exception as e:
            print(f"Error invoking the chain: {e}")
            return None



# bison32k.py
from .google import Google

class Bison32k(Google):
    name = "bison_32k"
    model = "chat-bison-32k"  # Override class variable

    def __init__(self, temperature=0.2, max_tokens=8192, **model_kwargs):
        super().__init__(**model_kwargs)

        self.temperature=temperature
        self.max_tokens=max_tokens

        # Override AUTHENTICATION per Model
        try:
            self.auth(self.__class__.name.lower())
        except Exception as e:
            print(f"Error during authentication: {e}")


        try:
            self.updateLLM()
        except Exception as e:
            print(f"Error updating LLM in {self.__class__.__name__}: {e}")




# bison.py
from .google import Google

class Bison(Google):
    name = "bison"
    model = "chat-bison"  # Override class variable

    def __init__(self, temperature=0.2, max_tokens=2048, **model_kwargs):
        super().__init__(**model_kwargs)

        self.temperature=temperature
        self.max_tokens=max_tokens
        
        # Override AUTHENTICATION per Model
        try:
            self.auth(self.__class__.name.lower())
        except Exception as e:
            print(f"Error during authentication: {e}")


        try:
            self.updateLLM()
        except Exception as e:
            print(f"Error updating LLM in {self.__class__.__name__}: {e}")




# llms.py
import importlib
import pkgutil

class LLMs:
    def __init__(self):
        pass

    @classmethod
    def auth(cls):
        pass

    def updateLLM(self):
        pass

    def run(self, prompt):
        pass

    @classmethod
    def list(cls):
        subclasses = cls.__subclasses__()
        subclass_names = [cls.name.lower() for cls in subclasses]
        return subclass_names



# coherecommandr.py
from .aws import AWS
import sys
from langchain_core.prompts import PromptTemplate

class CohereCommandR(AWS):
    name = "cohere_command_r"
    model = "cohere.command-r-v1:0"  # Override class variable
    chat = False # NOTE: Cohere Command does NOT support Chat

    def __init__(self, temperature=0.2, max_tokens=4096, **model_kwargs):
        super().__init__(**model_kwargs)

        self.model_kwargs = {
            "temperature": temperature,
            "max_tokens": max_tokens,
            **model_kwargs
        }

        # Override - Cohere Command R requires a different input template
        try:
            system_message = "You are a helpful assistant. Answer all questions to the best of your ability.\n{question}"
            self.prompt_template = PromptTemplate(
                input_variables=["question"], 
                template=system_message
            )
        except Exception as e:
            print(f"Error initializing prompt template: {e}")
            sys.exit(1)


        # Override AUTHENTICATION per Model
        try:
            self.auth(self.__class__.name.lower())
        except Exception as e:
            print(f"Error during authentication: {e}")


        try:
            self.updateLLM(self.chat)
        except Exception as e:
            print(f"Error updating LLM in {self.__class__.__name__}: {e}")


    # Override - Cohere Command R requires a different input template
    def run(self, prompt):
        print("OVERRIDE!")
        print(self.chain)
        try:
            response = self.chain.invoke(prompt)
            return response
        except Exception as e:
            print(f"Error invoking the chain: {e}")
            return None




# gpt35turbo.py
from .openai import OpenAI

class GPT35Turbo(OpenAI):
    name = "gpt_35_turbo"
    model = "gpt-3.5-turbo-1106"  # Override class variable

    def __init__(self, temperature=0.2, max_tokens=4096, **model_kwargs):
        super().__init__(**model_kwargs)
        self.temperature = temperature
        self.max_tokens = max_tokens

        # Override AUTHENTICATION per Model
        try:
            self.auth(self.__class__.name.lower())
        except Exception as e:
            print(f"Error during authentication: {e}")


        try:
            self.updateLLM()
        except Exception as e:
            print(f"Error updating LLM in {self.__class__.__name__}: {e}")




# aws/__init__.py
from .geminipro1 import GeminiPro1 as gemini_pro_1
from .geminipro15 import GeminiPro15 as gemini_pro_1_5
from .geminiflash15 import GeminiFlash15 as gemini_flash_1_5
from .bison import Bison as bison
from .bison32k import Bison32k as bison_32k
from .google import Google

__all__ = ['gemini_pro_1', 'gemini_pro_1_5', 'gemini_flash_1_5', 'bison', 'bison_32k']



# mistral7binstruct.py
from .aws import AWS

class Mistral7bInstruct(AWS):
    name = "mistral_7b_instruct"
    model = "mistral.mistral-7b-instruct-v0:2"  # Override class variable

    def __init__(self, temperature=0.2, max_tokens=8192, **model_kwargs):
        super().__init__(**model_kwargs)

        self.model_kwargs = {
            "temperature": temperature,
            "max_tokens": max_tokens,
            **model_kwargs
        }

        # Override AUTHENTICATION per Model
        try:
            self.auth(self.__class__.name.lower())
        except Exception as e:
            print(f"Error during authentication: {e}")

        try:
            self.updateLLM()
        except Exception as e:
            print(f"Error updating LLM in {self.__class__.__name__}: {e}")




# gemini.py
import os, sys, json
from google.oauth2 import service_account
from dotenv import load_dotenv

# https://cloud.google.com/vertex-ai/generative-ai/docs/multimodal/configure-safety-attributes
from langchain_google_vertexai import ChatVertexAI, HarmBlockThreshold, HarmCategory
from langchain_core.messages import HumanMessage, SystemMessage
from langchain_core.prompts import ChatPromptTemplate, MessagesPlaceholder
from langchain_core.output_parsers import StrOutputParser
from ..llms import LLMs

class Google(LLMs):
    name = "google" # Class variable
    model = ""  # Class variable
    temperature = 0.2  # Class variable
    max_tokens = None  # Class variable

    def __init__(self, **model_kwargs):
        super().__init__()

        self.model_kwargs = model_kwargs  # Store model kwargs

        # AUTHENTICATION
        try:
            self.auth()
        except Exception as e:
            print(f"Error during authentication: {e}")

        # Initialize the output parser here if it's going to be static
        self.output_parser = StrOutputParser()

        # If the prompt template is static, initialize it here
        try:
            self.prompt_template = ChatPromptTemplate.from_messages([
                SystemMessage(content="You are a helpful assistant. Answer all questions to the best of your ability."),
                MessagesPlaceholder(variable_name="messages"),
            ])
        except Exception as e:
            print(f"Error initializing prompt template: {e}")
            sys.exit(1)

        try:
            self.updateLLM()
        except Exception as e:
            print(f"Error updating LLM: {e}")
            sys.exit(1)

    def updateLLM(self):
        try:
            GCP_PROJECT_ID = os.environ.get("GCP_PROJECT_ID")
            GCP_REGION = os.environ.get("GCP_REGION")
            GCP_CREDENTIALS_JSON = os.environ.get("GCP_CREDENTIALS_JSON")
            credentials = service_account.Credentials.from_service_account_info(json.loads(GCP_CREDENTIALS_JSON))
            scoped_creds = credentials.with_scopes(["https://www.googleapis.com/auth/cloud-platform"])

            self.llm = ChatVertexAI(
                model_name=self.model,
                convert_system_message_to_human=False,
                project=GCP_PROJECT_ID,
                location=GCP_REGION,
                credentials=scoped_creds,
                max_output_tokens=self.max_tokens,
                temperature=self.temperature,
                model_kwargs=self.model_kwargs
            )

            self.chain = self.prompt_template | self.llm | self.output_parser

        except Exception as e:
            print(f"Error initializing LLM or chain: {e}")
            sys.exit(1)

    @classmethod
    def auth(cls, model=None):
        # Google auth happens here
        try:
            if(model):
                auth_status = load_dotenv(f".llms/google-{model}", override=True)
            else:
                auth_status = load_dotenv(".llms/google")
            if not auth_status:
                if not (
                    os.environ.get("GCP_PROJECT_ID") and
                    os.environ.get("GCP_REGION") and
                    os.environ.get("GCP_CREDENTIALS_JSON")
                ):
                    raise EnvironmentError("`GCP_PROJECT_ID` or `GCP_REGION` or `GCP_CREDENTIALS_JSON` env variables not defined, and `.llms/google` not available")

        except Exception as e:
            print(f"Error loading authentication for Google: {e}")
            sys.exit(1)

    def run(self, prompt):
        try:
            response = self.chain.invoke({
                "messages": [
                    HumanMessage(content=prompt),
                ],
            })
            return response
        except Exception as e:
            print(f"Error invoking the chain: {e}")
            return None



# setup.py
from setuptools import setup, find_packages

with open("requirements.txt") as f:
    required = f.read().splitlines()

setup(
    name="easy-llms",
    description="Easy '1-line' calling of every LLM from OpenAI, MS Azure, AWS Bedrock, GCP Vertex, and Ollama",
    version="0.1.9",
    author="Ventz Petkov",
    url="https://github.com/ventz/easy-llms",
    license="MIT",
    long_description=open("README.md").read(),
    long_description_content_type="text/markdown",
    packages=find_packages(exclude=["tests*"]),
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    python_requires=">=3.6",
    install_requires=required,
    include_package_data=True,
    #extras_require={"dev": ["pytest""]},
    # Coming soon!
    #test_suite="tests",
)



# geminipro15.py
from .google import Google
from langchain_google_vertexai import ChatVertexAI, HarmBlockThreshold, HarmCategory

class GeminiPro15(Google):
    name = "gemini_pro_1_5"
    model = "gemini-1.5-pro-preview-0514"  # Override class variable

    def __init__(self, temperature=0.2, max_tokens=8192, **model_kwargs):
        super().__init__(**model_kwargs)

        self.temperature=temperature
        self.max_tokens=max_tokens

        safety_settings={
            # https://readthedocs.org/projects/google-auth/downloads/pdf/latest/
            # https://ai.google.dev/docs/safety_setting_gemini
            # Not Present in Gemini
            #HarmCategory.HARM_CATEGORY_DEROGATORY: HarmBlockThreshold.BLOCK_NONE,
            #HarmCategory.HARM_CATEGORY_TOXICITY: HarmBlockThreshold.BLOCK_NONE,
            #HarmCategory.HARM_CATEGORY_VIOLENCE: HarmBlockThreshold.BLOCK_NONE,
            #HarmCategory.HARM_CATEGORY_SEXUAL: HarmBlockThreshold.BLOCK_NONE,
            #HarmCategory.HARM_CATEGORY_MEDICAL: HarmBlockThreshold.BLOCK_NONE,
            #HarmCategory.HARM_CATEGORY_DANGEROUS: HarmBlockThreshold.BLOCK_NONE,
            # Everything bellow present in Gemini
            HarmCategory.HARM_CATEGORY_UNSPECIFIED: HarmBlockThreshold.BLOCK_NONE,
            HarmCategory.HARM_CATEGORY_HARASSMENT: HarmBlockThreshold.BLOCK_NONE,
            HarmCategory.HARM_CATEGORY_HATE_SPEECH: HarmBlockThreshold.BLOCK_NONE,
            HarmCategory.HARM_CATEGORY_SEXUALLY_EXPLICIT: HarmBlockThreshold.BLOCK_NONE,
            HarmCategory.HARM_CATEGORY_DANGEROUS_CONTENT: HarmBlockThreshold.BLOCK_NONE
        }

        self.llm.safety_settings = safety_settings

        # Override AUTHENTICATION per Model
        try:
            self.auth(self.__class__.name.lower())
        except Exception as e:
            print(f"Error during authentication: {e}")


        try:
            self.updateLLM()
        except Exception as e:
            print(f"Error updating LLM in {self.__class__.__name__}: {e}")




# openai/__init__.py
from .gpt35turbo import GPT35Turbo as gpt_35_turbo
from .gpt35turbo16k import GPT35Turbo16k as gpt_35_turbo_16k
from .gpt4turbo import GPT4Turbo as gpt_4_turbo
from .gpt4o import GPT4o as gpt_4o
from .gpt4omini import GPT4oMini as gpt_4o_mini
from .gpt4 import GPT4 as gpt_4
from .gpt432k import GPT432k as gpt_4_32k
from .openai import OpenAI

__all__ = ['gpt_35_turbo', 'gpt_35_turbo_16k', 'gpt_4_turbo', 'gpt_4o', 'gpt_4o_mini', 'gpt_4', 'gpt_4_32k']
